/*---------------------------------------------------------------------------*\
 *           interactive networked Virtual Reality system (inVRs)            *
 *                                                                           *
 *    Copyright (C) 2005-2009 by the Johannes Kepler University, Linz        *
 *                                                                           *
 *                            www.inVRs.org                                  *
 *                                                                           *
 *              contact: canthes@inVRs.org, rlander@inVRs.org                *
\*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*\
 *                                License                                    *
 *                                                                           *
 * This library is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU Library General Public License as published    *
 * by the Free Software Foundation, version 2.                               *
 *                                                                           *
 * This library is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         *
 * Library General Public License for more details.                          *
 *                                                                           *
 * You should have received a copy of the GNU Library General Public         *
 * License along with this library; if not, write to the Free Software       *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 *
\*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*\
 *                           Project: FFD                                    *
 *                                                                           *
 * The FFD library was developed during a practical at the Johannes Kepler   *
 * University, Linz in 2009 by Marlene Hochrieser                            *
 *                                                                           *
\*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*\
 *                                Changes                                    *
 *                                                                           *
 *                                                                           *
 *                                                                           *
 *                                                                           *
\*---------------------------------------------------------------------------*/


#ifndef GLOBALDEFORMACTIONEVENT_H_INCLUDED
#define GLOBALDEFORMACTIONEVENT_H_INCLUDED

#include <gmtl/Matrix.h>

#include <inVRs/SystemCore/DataTypes.h>
#include <inVRs/SystemCore/EventManager/Event.h>
#include <inVRs/SystemCore/EventManager/EventFactory.h>

#include "ffd/Actions/GlobalDeformAction.h"
#include "ActionEvent.h"

/*******************************************************************************
 * @class GlobalDeformActionEvent
 * @brief Encodes, decodes, and executes the global deformation action.
 */
class GlobalDeformActionEvent : public ActionEvent, public GlobalDeformAction
{
    public:
        GlobalDeformActionEvent(unsigned int environmentBasedId, gmtl::Matrix44f
            dMatrix, bool center);
        GlobalDeformActionEvent();
        virtual ~GlobalDeformActionEvent();

        typedef EventFactory<GlobalDeformActionEvent> Factory;

        virtual void encode(NetMessage *message);
        virtual void decode(NetMessage *message);

        virtual void execute();

    private:
        unsigned int environmentBasedId;
};

#endif // GLOBALDEFORMACTIONEVENT_H_INCLUDED
